#---------------------------------#
#  Grid plot Demo for SIMRES Lib  #
#    Create By: Jian Li.         #
#---------------------------------#

#--------------------------------#
# Import everthing from tk & ttk #
#--------------------------------#
import tkinter as tk
import tkinter.ttk as ttk
import tkinter.messagebox

from tkinter import *
from tkinter.ttk import * 

from mpl_toolkits.axes_grid1.inset_locator import inset_axes, zoomed_inset_axes
from mpl_toolkits.axes_grid1.anchored_artists import AnchoredSizeBar


import matplotlib
matplotlib.use('TkAgg')
import numpy as np
from numpy import arange, sin, pi

#from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2TkAgg
from matplotlib.backends.backend_tkagg import (
    FigureCanvasTkAgg, NavigationToolbar2Tk)
# implement the default mpl key bindings
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure
# to use polygon
from matplotlib.patches import Circle, Wedge, Polygon
from matplotlib.collections import PatchCollection
from matplotlib import cm
import matplotlib.pyplot as plt

import simres as sim


def change_view_direction(self):
    global viewdir
    global dirList
    if dirList.get() == "TOP":
        viewdir = 0
    elif dirList.get() == "FRONT":
        viewdir = 1
    else:
        viewdir = 2
        
    update_layer_list()

def update_layer_list():
    global viewdir
    global dirList
    global layerList
    global NX, NY, NZ
    #change_view_direction()
    if viewdir == 0:
        layerList["values"] = [i for i in range(1,NZ+1)]
        layerList.current(0) 
    elif viewdir == 1:
        layerList["values"] = [i for i in range(1,NY+1)]
        layerList.current(0) 
    else:
        layerList["values"] = [i for i in range(1,NX+1)]
        layerList.current(0) 
    
def on_key_event(event):
    print('you pressed %s' % event.key)
    key_press_handler(event, canvas, toolbar)




def _quit():
    ret =tkinter.messagebox.askokcancel('提示', '确认退出吗？')
    if ret != True:
        return
    root.quit()     # stops mainloop
    root.destroy()  # this is necessary on Windows to prevent
                    # Fatal Python Error: PyEval_RestoreThread: NULL tstate
def update_view_coord(dir, coords, layerindex, patches):
    global NX, NY, NZ
    patches.clear()
    o2a = sim.gm_getv_o2a()
    if dir == 0:
        n2d = NX*NY
        lidx = layerindex - 1
        cidx = 0
        for i in range(n2d):
            cidx = lidx * n2d + i
            if o2a[cidx] < 0:
                continue
            points = np.ndarray(shape=(4,2), dtype=float)
            points[0][0] = coords[cidx][0][0]
            points[0][1] = coords[cidx][0][1]
            points[1][0] = coords[cidx][1][0]
            points[1][1] = coords[cidx][1][1]
            points[2][0] = coords[cidx][3][0]
            points[2][1] = coords[cidx][3][1]
            points[3][0] = coords[cidx][2][0]
            points[3][1] = coords[cidx][2][1]   
            polygon = Polygon(points, True)
            patches.append(polygon)

    elif dir == 1:
        n2d = NX*NY
        j = layerindex - 1
        cidx = 0
        for k in range(NZ):
            for i in range(NX):
                cidx = k * n2d + j * NX + i
                if o2a[cidx] < 0:
                    continue
                points = np.ndarray(shape=(4,2), dtype=float)
                points[0][0] = coords[cidx][0][0]
                points[0][1] = coords[cidx][0][2]
                points[1][0] = coords[cidx][1][0]
                points[1][1] = coords[cidx][1][2]
                points[2][0] = coords[cidx][5][0]
                points[2][1] = coords[cidx][5][2]
                points[3][0] = coords[cidx][4][0]
                points[3][1] = coords[cidx][4][2]   
                polygon = Polygon(points, True)
                patches.append(polygon)
        
    else:
        n2d = NX*NY
    
def update_view_property(dir, layerindex, propname, prop):
    global NX, NY, NZ
    prop.clear()
    print('get property ' + propname + ' data from layer ' + str(layerIndex))
    tmp_prop = sim.gm_getv_gp(propname)
    if len(tmp_prop) <= 0:
        tmp_prop = sim.gm_getv_rp(propname)
    
    print(len(tmp_prop))
    o2a = sim.gm_getv_o2a()
    if dir == 0:
        n2d = NX*NY
        lidx = layerindex - 1
        cidx = 0
        for i in range(n2d):
            cidx = lidx * n2d + i
            if o2a[cidx] < 0:
                continue
            prop.append(tmp_prop[o2a[cidx]])

    elif dir == 1:
        n2d = NX*NY
        n2d = NX*NY
        j = layerindex - 1
        cidx = 0
        for k in range(NZ):
            for i in range(NX):
                cidx = k * n2d + j * NX + i
                if o2a[cidx] < 0:
                    continue
                prop.append(tmp_prop[o2a[cidx]])
                
    else:
        n2d = NX*NY
                    
def plotGrid():
    global layerIndex
    global coord
    global patches
    global cellno
    global xmax
    global xmin
    global ymax
    global ymin
    global coords
    global NX, NY, NZ
    prop = []
    #global plt
    if layerIndex<0:
        layerIndex = int(layerList.get())
        isUpdateCoord = True
    elif layerIndex != int(layerList.get()):
        layerIndex = int(layerList.get())
        isUpdateCoord = True
    else:
        isUpdateCoord = False
        
    propname = propList.get()

    if isUpdateCoord:
        print('Update Coords data from layer ' + str(layerIndex))
        #coord = sim.gm_slice_coord(layerIndex)
        if coords == []:
            print("Empty array, may be a complete inactive layer.")
            return 0
        else:
            update_view_coord(viewdir, coords, layerIndex, patches)
                
                
    update_view_property(viewdir, layerIndex, propname, prop)            
    # update property
    global measureSystem
    isDrawBorder = measureSystem.get()
    #print(measureSystem)

    global fig
    global plt
    fig.clf()
    ax = fig.add_subplot(1,1,1)
    global canvas
    plt.set_cmap(my_cmap)  # my_cmap can be used here

    #prop = sim.gm_prop_slice(propname,layerIndex) 
    
    print(len(prop))   
    print(cellno)
    upper = cellno
    #colors = [i for i in range(0, upper)]
    # v=40是为了测试颜色与值映射时改变值最大值引起的变化
    # colors[upper - 1]= 40
    ### 网格的边框颜色在这里设置
    if isDrawBorder:
        p = PatchCollection(patches,edgecolor='#000000',alpha=1.0)
    else:
        p = PatchCollection(patches,alpha=1.0)
    #isDrawBorder = not isDrawBorder
    p.set_array(np.array(prop))
    ax.add_collection(p)
    #if isUpdateCoord:
    deltax = abs(xmax - xmin) * 0.05
    deltay = abs(ymax - ymin) * 0.05
    print('Boundary box:')
    print(str(xmin) + "," + str(xmax))
    print(str(ymin) + "," + str(ymax))
    #plt.ylim(ymin=ymin - deltay,ymax=ymax + deltay)
    #plt.xlim(xmin=xmin - deltax,xmax=xmax + deltax)
    #ax.set_ylim( ymin - deltay, ymax + deltay )  
    #ax.set_xlim( xmin - deltax, xmax + deltax )
    ax.autoscale()
    # 用来设置颜色与值映射的最大值最小值，等于对色标进行上下值限制
    #p.set_clim(vmin=0,vmax=0.3)
    colorBar = fig.colorbar(p, ax=ax ) #,extend='both')
    colorBar.set_label(propname + ' Map')
    ax.set_title(propname + " K=" + str(layerIndex))
    ax.set_aspect('equal',adjustable='datalim')

    #ax2 = fig.add_subplot(1,2,2)
    #ax2.hist(prop)
    #canvas.show()
    canvas.draw()
    
if __name__ == "__main__":   
    #--------------------------------
    # Load data  
    #--------------------------------
    files = [r'E:\data\MRSRESULT\SPE9_S.data']
    ret = sim.load_result(files[0],"u","ei")
    gmdims = sim.gm_dims()
    maxK = gmdims[2]
    NX = gmdims[0]
    NY = gmdims[1]
    NZ = gmdims[2]
    layerIndex = -1

    coords = sim.gm_getv_coords()


    gmPropertySelections = {"none" : 0}
    gmgp = sim.gm_getn_gp()
    gmrp = sim.gm_getn_rp()
    gmap = gmgp + gmrp
    print('checking properties...')
    for name in gmap:
        gmPropertySelections[name] = 0
        print(name)

    coord = [] # sim.gm_slice_coord(layerIndex)
    cellno = 0
    patches = []    
    xmax = 0.0
    xmin = 0.0
    ymax = 0.0
    ymin = 0.0
    # 0: K- , 1: J+, 2: I-
    viewdir = 0

    import sys
    if sys.version_info[0] < 3:
        import Tkinter as tk
    else:
        import tkinter as tk

    root = tk.Tk()
    root.wm_title("Simulation Resul Viewer - Plot Grid")

    cdict2 = {'red': ((0.0, 0.0, 0.0),
                      (0.25,0.0, 0.0),
                      (0.5, 0.0, 0.0),
                      (0.75,1.0, 1.0),
                      (1.0, 1.0, 1.0)),
             'green': ((0.0, 0.0, 0.0),
                       (0.25,1.0, 1.0),
                       (0.5, 1.0, 1.0),
                       (0.75,1.0, 1.0),
                       (1.0, 0.0, 1.0)),
             'blue': ((0.0, 0.0, 1.0),
                      (0.25,1.0, 1.0),
                      (0.5, 0.0, 0.0),
                      (0.75,0.0, 0.0),
                      (1.0, 0.0, 1.0))}
    my_cmap = matplotlib.colors.LinearSegmentedColormap('my_colormap',cdict2,256)
    cm.register_cmap(cmap=my_cmap)

    fig = Figure(figsize=(5, 4), dpi=100)
    #a = f.add_subplot(111)
    #fig, ax = plt.subplots()
    #t = arange(0.0, 3.0, 0.01)
    #s = sin(2*pi*t)

    #ax.plot(t, s)


    # a tk.DrawingArea
    canvas = FigureCanvasTkAgg(fig, master=root)
    # canvas.show()
    canvas.draw()
    canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)

    #toolbar = NavigationToolbar2TkAgg(canvas, root)
    toolbar = NavigationToolbar2Tk(canvas, root)
    toolbar.update()
    canvas._tkcanvas.pack(side=tk.TOP, fill=tk.BOTH, expand=1) 
    button = tk.Button(master=root, text='Quit', command=_quit)
    button.pack(side=tk.BOTTOM, fill=tk.BOTH)

    label = Label(root,text = '属性')
    label.pack(side=tk.LEFT)
    #name = StringVar()
    propList=ttk.Combobox(root,width=10)#, textvariable=name)
    propList["state"] = "readonly"
    propList.pack(side=tk.LEFT)
    propList["values"] = gmap
    propList.current(0) 

    label = Label(root,text = '层号')
    label.pack(side=tk.LEFT)

    layerList=ttk.Combobox(root,width=5)
    layerList["state"] = "readonly"
    layerList["values"] = [i for i in range(1,NZ+1)]
    layerList.current(0) 
    layerList.pack(side=tk.LEFT)

    dirList =ttk.Combobox(root,width=5)
    dirList["state"] = "readonly"
    dirList["values"] = ["TOP","FRONT","LEFT"]
    dirList.current(0) 
    dirList.pack(side=tk.LEFT)
    dirList.bind("<<ComboboxSelected>>",change_view_direction)

    update_layer_list()

    measureSystem = IntVar()
    chkPlotBorder = ttk.Checkbutton(root, text='Border', 
            command=plotGrid, variable=measureSystem,
            onvalue=True, offvalue=False)
    chkPlotBorder.pack(side=tk.LEFT)
            
    button = tk.Button(master=root, text='Plot', command=plotGrid)
    button.pack(side=tk.LEFT)
    canvas.mpl_connect('key_press_event', on_key_event)
    tk.mainloop()
# If you put ro