"""
 Program Name     : A Fast GUI for Simulation Reseult Plot
 Created By       : Jian Li
 Created on       : 30th Jan, 2017
 Last Modified By : Jian Li
 Last Modified On : 30th Aug, 2022
 * All Right Reserved By the auther thecolors@126.com
 * This is a demostration for SimResLib. A Free License mode 
 * only for non-commercial use. 
 * 
"""
# --------------------------------------------------------------#
# Import everthing from tk & ttk                               #
# --------------------------------------------------------------#
import tkinter as tk
import tkinter.ttk as ttk
from tkinter import *
from tkinter import messagebox
from tkinter.ttk import *
# --------------------------------------------------------------#
# Import filedialog from tk                                    #
# --------------------------------------------------------------#
import tkinter.filedialog as filedialog

import simres as sim

# --------------------------------------------------------------#
# Import matplotlib                                            #
# --------------------------------------------------------------#
import matplotlib

matplotlib.use('TkAgg')
from numpy import arange, sin, pi
# Python35
# from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2TkAgg
# Python37
from matplotlib.backends.backend_tkagg import (
    FigureCanvasTkAgg, NavigationToolbar2Tk)
# implement the default mpl key bindings
from matplotlib.backend_bases import key_press_handler
from matplotlib.figure import Figure

# --------------------------------------------------------------#
# Import some system modules                                   #
# --------------------------------------------------------------#
import os
import math
# --------------------------------------------------------------#
# To Implement a dynamic array with key                        #
# --------------------------------------------------------------#
from collections import defaultdict


def free_tree():
    return defaultdict(free_tree)


class SimResMainFrame(tk.Frame):
    propertySelections = {"none": 0}  # replace the free_tree()
    caseSelections = {"none": 0}
    objectSelections = {"G_FIELD": 0}
    paramSelections = {"G_time": 0, 'G_list': 0}
    groupParentId = 'groups00000'
    wellParentId = 'wells00000'
    aquiferParentId = 'aquifers00000'
    page_number = 1

    isUnifout = True
    plotCount = 0
    maxPlotCol = 4

    def __init__(self, master=None):
        super().__init__(master)
        # self.pack()
        # The argument sticky=tk.N+tk.S+tk.E+tk.aW to self.grid() is necessary so that 
        # the Application widget will expand to fill its cell of the top-level window's grid. 
        # self.rowconfigure(0, weight=1)
        self.initImages()
        self.createMenus()
        self.createMainDisplay()
        # self.grid(sticky=tk.N+tk.S+tk.E+tk.W)
        self.pack(expand=tk.YES, fill=tk.BOTH)

    def createMenus(self):
        self.mu_main = tk.Menu()
        # File menu
        self.mu_file = tk.Menu(self.mu_main, tearoff=0)
        self.mu_file.add_command(label="打开...", command=self.openFile)
        self.mu_file.add_command(label="打印...", command=self.printFile)
        self.mu_file.add_separator()
        self.mu_file.add_command(label="退出", command=self.exitProgram)
        self.mu_main.add_cascade(label="文件", menu=self.mu_file)
        # 设置菜单
        self.mu_setting = tk.Menu(self.mu_main, tearoff=0)
        self.mu_setting.add_checkbutton(label="UNFOUT格式", command=self.checkUnifout)
        self.mu_setting.add_separator()
        self.mu_setting.add_command(label="取消[模型]选择", command=self.clearCaseSelection)
        self.mu_setting.add_command(label="取消[参数]选择", command=self.clearPropertySelection)
        self.mu_setting.add_command(label="取消[对象]选择", command=self.clearObjectSelection)
        self.mu_setting.add_separator()
        self.mu_setting.add_command(label="取消所有数据的选择", command=self.clearAllSelection)
        self.mu_main.add_cascade(label="设置", menu=self.mu_setting)

        # self.mu_setting.invoke(self.mu_setting.index('UNFOUT格式'))

        # Help Menu
        self.mu_help = tk.Menu(self.mu_main, tearoff=0)
        self.mu_help.add_command(label="显示帮助...", command=self.showHelp)
        self.mu_help.add_command(label="关于...", command=self.showAbout)
        self.mu_main.add_cascade(label="帮助", menu=self.mu_help)
        # Link Menu
        self.master.config(menu=self.mu_main)

    def createMainDisplay(self):
        print("createMainDisplay called")
        # ------------------------------------------------------------
        # Case tree
        # ------------------------------------------------------------
        self.frame1 = ttk.Frame(self.master, width=150, height=600)
        # propagate(0) is used to force to set a width for a widget
        self.frame1.propagate(0)
        label1 = Label(self.frame1, text='模型')
        label1.pack()
        self.caseTree = ttk.Treeview(self.frame1, selectmode='extended', show="tree")  # ,height=30)
        # self.caseTree.grid(row=0, column=0,rowspan = 3, sticky=tk.N+tk.S+tk.W)
        self.addVSBarToWidget(self.frame1, self.caseTree)
        self.addHSBarToWidget(self.frame1, self.caseTree)
        self.caseTree.pack(side=tk.LEFT, fill=tk.Y)

        self.frame1.pack(side=tk.LEFT, fill=tk.Y)
        self.caseTree.bind("<Button-1>", self.caseTreeClick)
        # ------------------------------------------------------------
        # Property tree
        # ------------------------------------------------------------
        self.frame2 = ttk.Frame(self.master, width=150, height=600)
        self.frame2.propagate(0)
        label2 = Label(self.frame2, text='参数')
        label2.pack()
        self.propertyTree = ttk.Treeview(self.frame2, selectmode='extended', show="tree")
        # Initialize property list
        list = self.getPropertyList()
        last_parent = ""
        for item in list:
            if item[1] != last_parent:
                parentItem = self.propertyTree.insert('', 'end', text=item[1], open=True)
                last_parent = item[1]
            self.propertyTree.insert(parentItem, 'end', iid=item[0], text=item[2], tag=item[0])
            self.propertyTree.tag_configure(item[0], image=self.img_unchecked)
            self.propertySelections[item[0]] = 0
        # self.propertyTree.grid(row=0, column=1,rowspan = 3,sticky=tk.N+tk.S+tk.W)
        self.addVSBarToWidget(self.frame2, self.propertyTree)
        # self.addHSBarToWidget(self.frame2,self.propertyTree)
        self.propertyTree.pack(side=tk.LEFT, fill=tk.Y)

        self.frame2.pack(side=tk.LEFT, fill=tk.Y)
        self.propertyTree.bind("<Button-1>", self.propertyTreeClick)

        # ------------------------------------------------------------
        # Object tree
        # ------------------------------------------------------------
        self.frame3 = ttk.Frame(self.master, width=150, height=600)
        # propagate(0) is used to force to set a width for a widget
        self.frame3.propagate(0)
        label3 = Label(self.frame3, text='对象')
        label3.pack()
        self.objectTree = ttk.Treeview(self.frame3, selectmode='extended', show="tree")  # ,height=30)
        parentNode = self.objectTree.insert('', 'end', iid=self.groupParentId, text="井组", open=True)
        self.objectTree.insert(parentNode, 'end', iid='G_FIELD', text="全区", tag='G_FIELD')
        self.objectTree.tag_configure('G_FIELD', image=self.img_unchecked)
        self.objectTree.insert('', 'end', iid=self.wellParentId, text="井", open=True)
        self.objectTree.insert('', 'end', iid=self.aquiferParentId, text="水体", open=True)
        # self.caseTree.grid(row=0, column=0,rowspan = 3, sticky=tk.N+tk.S+tk.W)
        self.addVSBarToWidget(self.frame3, self.objectTree)
        self.objectTree.pack(side=tk.LEFT, fill=tk.Y)
        self.frame3.pack(side=tk.LEFT, fill=tk.Y)
        self.objectTree.bind("<Button-1>", self.objectTreeClick)

        # ------------------------------------------------------------
        # Display area
        # ------------------------------------------------------------
        s = Style()
        s.configure('My.TFrame', background='#ffffff')
        self.frame4 = ttk.Frame(self.master, width=500, height=600, style='My.TFrame')
        self.frame4.propagate(0)
        self.updateButton = tk.Button(self.frame4, text='绘图', command=self.drawPlot, relief=tk.RAISED)
        self.updateButton.pack(side=tk.TOP, fill=tk.X)

        self.drawRightButton = tk.Button(self.frame4, text='>', command=self.get_right_page_nu, relief=tk.RAISED,
                                         width=3, height=1)
        # self.drawRightButton.grid(row=0, column=1)
        self.drawRightButton.pack(anchor=S, fill=tk.NONE, side=tk.RIGHT)

        self.drawLeftButton = tk.Button(self.frame4, text='<', command=self.get_left_page_nu, relief=tk.RAISED,
                                        width=3,height=1)
        # self.drawLeftButton.grid(row=0,column=0)
        self.drawLeftButton.pack(anchor=S, fill=tk.NONE, side=tk.RIGHT)



        # put a plot area
        self.f = Figure(figsize=(5, 4), dpi=100)
        self.canvas = FigureCanvasTkAgg(self.f, master=self.frame4)

        # self.canvas.show()
        self.canvas.draw()

        self.canvas.get_tk_widget().pack(expand=tk.YES, side=tk.BOTTOM, fill=tk.BOTH)
        # reserved for  pyplot
        self.frame4.pack(expand=tk.YES, side=tk.RIGHT, fill=tk.BOTH)

        # ------------------------------------------------------------
        # params tree paramTree
        # ------------------------------------------------------------
        self.frame5 = ttk.Frame(self.master, width=150, height=600)
        # propagate(0) is used to force to set a width for a widget
        self.frame5.propagate(0)
        label5 = Label(self.frame5, text='X轴')
        label5.pack()
        self.paramTree = ttk.Treeview(self.frame5, selectmode='extended', show="tree")  # ,height=30)
        parentNode = self.paramTree.insert('', 'end', iid=self.groupParentId, text="图像X轴", open=True)
        self.paramTree.insert(parentNode, 'end', iid='G_list', text="list", tag='G_list')
        self.paramTree.tag_configure('G_list', image=self.img_unchecked)
        self.paramTree.insert(parentNode, 'end', iid='G_time', text="时间", tag='G_time')
        self.paramTree.tag_configure('G_time', image=self.img_unchecked)

        # self.caseTree.grid(row=0, column=0,rowspan = 3, sticky=tk.N+tk.S+tk.W)
        self.addVSBarToWidget(self.frame5, self.paramTree)
        self.paramTree.pack(side=tk.LEFT, fill=tk.Y)
        self.frame5.pack(side=tk.LEFT, fill=tk.Y)
        self.paramTree.bind("<Button-1>", self.paramTreeClick)

    def objectTreeClick(self, event):
        iid = self.objectTree.identify_row(event.y)
        if iid != "" and (iid in self.objectSelections):
            print(iid + ":", end="")
            print(str(self.objectSelections[iid]) + "=>", end="")
            # toggle and add is almost the same in this case
            self.objectTree.selection_toggle(iid)
            if self.objectSelections[iid] == 0:
                self.objectSelections[iid] = 1
                self.objectTree.tag_configure(iid, image=self.img_checked)
            else:
                self.objectSelections[iid] = 0
                self.objectTree.tag_configure(iid, image=self.img_unchecked)
            print(self.objectSelections[iid])
            print(1)

    def caseTreeClick(self, event):
        iid = self.caseTree.identify_row(event.y)
        if iid != "" and (iid in self.caseSelections):
            print(iid + ":", end="")
            print(str(self.caseSelections[iid]) + "=>", end="")
            # toggle and add is almost the same in this case
            self.caseTree.selection_toggle(iid)
            if self.caseSelections[iid] == 0:
                self.caseSelections[iid] = 1
                self.caseTree.tag_configure(iid, image=self.img_checked)
            else:
                self.caseSelections[iid] = 0
                self.caseTree.tag_configure(iid, image=self.img_unchecked)
            print(self.caseSelections[iid])
            print(2)

    def propertyTreeClick(self, event):
        iid = self.propertyTree.identify_row(event.y)
        if iid != "" and (iid in self.propertySelections):
            # this is a test to check is the value is already inside list
            # print(type(self.propertySelections[iid]) == int)
            print(iid + ":", end="")
            print(str(self.propertySelections[iid]) + "=>", end="")
            # toggle and add is almost the same in this case
            self.propertyTree.selection_toggle(iid)
            if self.propertySelections[iid] == 0:
                self.propertySelections[iid] = 1
                self.propertyTree.tag_configure(iid, image=self.img_checked)
            else:
                self.propertySelections[iid] = 0
                self.propertyTree.tag_configure(iid, image=self.img_unchecked)
            print(self.propertySelections[iid])
            print(3)
        # else:
        #    print("Nothing selected")

    def paramTreeClick(self, event):
        iid = self.paramTree.identify_row(event.y)
        if iid != "" and (iid in self.paramSelections):
            # this is a test to check is the value is already inside list
            # print(type(self.propertySelections[iid]) == int)
            print(iid + ":", end="")
            print(str(self.paramSelections[iid]) + "=>", end="")
            # toggle and add is almost the same in this case
            self.paramTree.selection_toggle(iid)
            if self.paramSelections[iid] == 0:
                self.paramSelections[iid] = 1
                self.paramTree.tag_configure(iid, image=self.img_checked)
            else:
                self.paramSelections[iid] = 0
                self.paramTree.tag_configure(iid, image=self.img_unchecked)
            print(self.paramSelections[iid])
            print(4)

    def openFile(self):
        print("openFile called")
        filename = filedialog.askopenfilename(filetypes=[("*.data", "*.data"), ("*.SMSPEC", "*.SMSPEC"),
                                                         ("*.rsm", "*.rsm"), ("*.prsm", "*.prsm")])
        print(filename)
        ext_str = os.path.splitext(filename)

        if ext_str[1].lower() == ".rsm" or ext_str[1].lower() == ".prsm":
            ret = sim.load_rsm(filename)
            if ret < 0:
                print("Open file error: " + str(ret))
                return
        else:
            flag = "m"
            if self.isUnifout:
                flag = "u"
            ret = sim.load_result(filename, flag, "s")
            if ret < 0:
                print("Open file error: " + str(ret))
                messagebox.showinfo('Open file error', '可能未勾选UNFOUT格式')
                return
        # make case ID 
        id = sim.get_label() + "_" + str(sim.default())
        print(id)
        self.caseTree.insert('', 'end', iid=id, text=id, tag=id)
        self.caseTree.tag_configure(id, image=self.img_unchecked)
        self.caseSelections[id] = 0
        return self.updateObjects()

    def updateObjects(self):
        # ------------------------------------------------------------
        # check groups
        # ------------------------------------------------------------
        groups = sim.sm_getn_groups()
        if len(groups) != 0:
            for gn in groups:
                id = "G_" + gn
                if id not in self.objectSelections:
                    self.objectSelections[id] = 0
                    self.objectTree.insert(self.groupParentId, 'end', iid=id, text=gn, tag=id)
                    self.objectTree.tag_configure(id, image=self.img_unchecked)
        # ------------------------------------------------------------
        # check wells
        # ------------------------------------------------------------
        wells = sim.sm_getn_wells()
        print(wells)
        if len(wells) != 0:
            for wn in wells:
                id = "W_" + wn
                if id not in self.objectSelections:
                    self.objectSelections[id] = 0
                    self.objectTree.insert(self.wellParentId, 'end', iid=id, text=wn, tag=id)
                    self.objectTree.tag_configure(id, image=self.img_unchecked)
        # ------------------------------------------------------------
        # check aquifers
        # ------------------------------------------------------------
        aquifers = sim.sm_getn_aquifers()
        if len(aquifers) != 0:
            for aqui in aquifers:
                id = "A_" + aqui
                if id not in self.objectSelections:
                    self.objectSelections[id] = 0
                    self.objectTree.insert(self.aquiferParentId, 'end', iid=id, text=aqui, tag=id)
                    self.objectTree.tag_configure(id, image=self.img_unchecked)

    def printFile(self):
        print("printFile called")

    def exitProgram(self):
        print("exitProgram called")
        root.quit()

    def showHelp(self):
        print("showHelp called")

    def showAbout(self):
        print("showAbout called")

    def getPropertyList(self):
        list = [("rate_oil", "流量", "产油量"), ("rate_water", "流量", "产水量"), ("rate_gas", "流量", "产气量"),
                ("rate_liquid", "流量", "产液量"),
                ("rate_waterinj", "流量", "注水量"),
                ("ratio_wct", "比率", "含水率"), ("ratio_gor", "比率", "气油比"), ("ratio_ogr", "比率", "油气比"),
                ("ratio_wgr", "比率", "水气比"),
                ("pr_apr", "压力", "平均压力"), ("pr_bhp", "压力", "井底流压"), ("pr_thp", "压力", "井口压力")]
        return list

    def initImages(self):
        # self.img_checked = ImageTk.PhotoImage(Image.open('checked_s4.ico'))
        # self.img_unchecked = ImageTk.PhotoImage(Image.open('unchecked_s4.ico'))
        self.img_checked = tk.PhotoImage(file='checked_s4.gif')
        self.img_unchecked = tk.PhotoImage(file='unchecked_s4.gif')

    def addVSBarToWidget(self, frame, w):
        vsb = ttk.Scrollbar(frame, orient="vertical", command=w.yview)
        w.configure(yscrollcommand=vsb.set)
        vsb.pack(side=tk.RIGHT, fill=tk.Y)

    def addHSBarToWidget(self, frame, w):
        hsb = ttk.Scrollbar(frame, orient="horizontal", command=w.xview)
        w.configure(xscrollcommand=hsb.set)
        hsb.pack(side=tk.BOTTOM, fill=tk.X)

    def checkUnifout(self):
        self.isUnifout = not self.isUnifout
        # print(self.isUnifout)

    def get_left_page_nu(self):
        self.page_number -= 1
        print(self.page_number, 'sssssss')
        return self.drawPageObjects()

    def get_right_page_nu(self):
        self.page_number += 1
        print(self.page_number, 'aaaaa')
        return self.drawPageObjects()

    def drawPlot(self):
        return self.drawByObjects()

    def drawByObjects(self):
        self.plotCount = 0
        for objectKey, objectValue in self.objectSelections.items():
            print(objectKey)
            if objectValue == 1:
                self.plotCount = self.plotCount + 1
        self.f.clf()
        print(str(self.plotCount) + "objects selected")

        plot_count = 0
        rows = 1
        d_val = math.sqrt(self.plotCount)
        cols = int(d_val)  # 图形列数
        if d_val > cols:
            cols = cols + 1
        rows = cols

        for objectKey, objectValue in self.objectSelections.items():
            if objectValue == 1:
                plot_count = plot_count + 1
                ax1 = self.f.add_subplot(rows, cols, plot_count)
                # print(str(row_index)+ ":"+str(col_index)+":"+str(plot_count))
                objectType = objectKey[0]
                objectName = objectKey[2:]
                print("Type: " + objectType + " : " + objectName)
                # begin to generate 1 plot
                for caseKey, caseValue in self.caseSelections.items():
                    if caseValue == 1:
                        tmpKey = caseKey.split('_')
                        caseID = int(tmpKey[len(tmpKey) - 1])
                        print("case ID : " + str(caseID))
                        paramName = ""
                        sim.as_default(caseID)
                        tv = sim.sm_getv_time()
                        for paramKey, paramValue in self.paramSelections.items():
                            if paramKey == 'G_time' and paramValue == 1:
                                tv = sim.sm_getv_time()
                                break
                            elif paramKey == 'G_list' and paramValue == 1:
                                tv = range(0, 41)
                                print("use list_")

                        for propertyKey, propertyValue in self.propertySelections.items():
                            if propertyValue == 1:
                                tmpKey = propertyKey.split('_')
                                propertyType = tmpKey[0]
                                propertyID = tmpKey[1]
                                if propertyType == "rate":
                                    if propertyID == "oil":
                                        paramName = "OPR"
                                    elif propertyID == "water":
                                        # well or other
                                        if objectType == 'A':
                                            paramName = "AQR"
                                        else:
                                            paramName = "WPR"
                                    elif propertyID == "waterinj":
                                        paramName = "WIR"
                                    elif propertyID == "gas":
                                        paramName = "GPR"
                                    elif propertyID == "liquid":
                                        paramName = "LPR"
                                elif propertyType == "ratio":
                                    if propertyID == "wct":
                                        paramName = "WCT"
                                    elif propertyID == "gor":
                                        paramName = "GOR"
                                    elif propertyID == "ogr":
                                        paramName = "OGR"
                                    elif propertyID == "wgr":
                                        paramName = "WGR"
                                elif propertyType == "pr":
                                    if propertyID == "bhp":
                                        paramName = "BHP"
                                    elif propertyID == "thp":
                                        paramName = "THP"
                                    elif propertyID == "apr":
                                        if objectType == 'A':
                                            paramName = "AQP"
                                        else:
                                            paramName = "PR"
                                if objectName != "FIELD":
                                    paramName = objectType + paramName
                                else:
                                    paramName = "F" + paramName
                                print(paramName)
                                # add a loop for historical data plot
                                vv = []
                                if objectType == "G":
                                    vv = sim.sm_getv_group(objectName, paramName)
                                elif objectType == "W":
                                    vv = sim.sm_getv_well(objectName, paramName)
                                elif objectType == "A":
                                    vv = sim.sm_getv_aquifer(objectName, paramName)
                                if len(vv) == len(tv):
                                    ax1.plot(tv, vv, label=caseKey + ":" + paramName)
                                    ax1.set_title(objectName)
                                    if plot_count == 1:
                                        ax1.legend()
                                else:
                                    # print(len(tv), len(vv))
                                    print('not same size(vv,tv)')
                # End of [for caseKey,caseValue in self.caseSelections.items():]
                # self.canvas.show()
                self.canvas.draw()

    def tk_lpage_warning(self):
        messagebox.showinfo('左翻页失败', '已经为第一页,重定向至第一页')

    def tk_rpage_warning(self):
        messagebox.showinfo('右翻页失败', '已经为最后一页,重定向至最后一页')

    def drawPageObjects(self):
        self.plotCount = 0
        for objectKey, objectValue in self.objectSelections.items():
            print(objectKey)
            if objectValue == 1:
                self.plotCount = self.plotCount + 1
        self.f.clf()
        print(str(self.plotCount) + "objects selected")

        plot_count = 0
        rows = 2
        cols = 2
        page_nu = math.ceil(self.plotCount / (rows * cols))
        if self.page_number < 0:
            self.tk_lpage_warning()
            self.page_number = 0
        elif self.page_number + 1 > page_nu:
            self.tk_rpage_warning()
            self.page_number = page_nu - 1

        print(page_nu, self.page_number + 1, '总页数--当前页')

        # cols = int(d_val)  # 图形列数
        # if d_val > cols:
        #     cols = cols + 1
        # rows = cols
        start = rows * cols * self.page_number
        end = rows * cols * (self.page_number + 1)
        if end > self.plotCount:
            end = self.plotCount
        print(start, end, "start----end")
        plot_list = [(k, v) for k, v in self.objectSelections.items() if v == 1]

        for objectdata in plot_list[start:end]:
            print(objectdata, 'objectdata')
            if objectdata[1] == 1:
                plot_count = plot_count + 1
                ax1 = self.f.add_subplot(rows, cols, plot_count)
                # print(str(row_index)+ ":"+str(col_index)+":"+str(plot_count))
                objectType = objectdata[0][0]
                objectName = objectdata[0][2:]
                print("Type: " + objectType + " : " + objectName)
                # begin to generate 1 plot
                for caseKey, caseValue in self.caseSelections.items():
                    if caseValue == 1:
                        tmpKey = caseKey.split('_')
                        caseID = int(tmpKey[len(tmpKey) - 1])
                        print("case ID : " + str(caseID))
                        paramName = ""
                        sim.as_default(caseID)
                        tv = sim.sm_getv_time()
                        for paramKey, paramValue in self.paramSelections.items():
                            if paramKey == 'G_time' and paramValue == 1:
                                tv = sim.sm_getv_time()
                                break
                            elif paramKey == 'G_list' and paramValue == 1:
                                tv = range(0, 41)
                                print("use list_")

                        for propertyKey, propertyValue in self.propertySelections.items():
                            if propertyValue == 1:
                                tmpKey = propertyKey.split('_')
                                propertyType = tmpKey[0]
                                propertyID = tmpKey[1]
                                if propertyType == "rate":
                                    if propertyID == "oil":
                                        paramName = "OPR"
                                    elif propertyID == "water":
                                        # well or other
                                        if objectType == 'A':
                                            paramName = "AQR"
                                        else:
                                            paramName = "WPR"
                                    elif propertyID == "waterinj":
                                        paramName = "WIR"
                                    elif propertyID == "gas":
                                        paramName = "GPR"
                                    elif propertyID == "liquid":
                                        paramName = "LPR"
                                elif propertyType == "ratio":
                                    if propertyID == "wct":
                                        paramName = "WCT"
                                    elif propertyID == "gor":
                                        paramName = "GOR"
                                    elif propertyID == "ogr":
                                        paramName = "OGR"
                                    elif propertyID == "wgr":
                                        paramName = "WGR"
                                elif propertyType == "pr":
                                    if propertyID == "bhp":
                                        paramName = "BHP"
                                    elif propertyID == "thp":
                                        paramName = "THP"
                                    elif propertyID == "apr":
                                        if objectType == 'A':
                                            paramName = "AQP"
                                        else:
                                            paramName = "PR"
                                if objectName != "FIELD":
                                    paramName = objectType + paramName
                                else:
                                    paramName = "F" + paramName
                                print(paramName)
                                # add a loop for historical data plot
                                vv = []
                                if objectType == "G":
                                    vv = sim.sm_getv_group(objectName, paramName)
                                elif objectType == "W":
                                    vv = sim.sm_getv_well(objectName, paramName)
                                elif objectType == "A":
                                    vv = sim.sm_getv_aquifer(objectName, paramName)
                                if len(vv) == len(tv):
                                    ax1.plot(tv, vv, label=caseKey + ":" + paramName)
                                    ax1.set_title(objectName)
                                    if plot_count == 1:
                                        ax1.legend()
                                else:
                                    # print(len(tv), len(vv))
                                    print('not same size(vv,tv)')
                # End of [for caseKey,caseValue in self.caseSelections.items():]
                # self.canvas.show()
                self.canvas.draw()

    def clearPropertySelection(self):
        for propertyKey, propertyValue in self.propertySelections.items():
            if propertyValue == 1:
                self.propertySelections[propertyKey] = 0
                self.propertyTree.tag_configure(propertyKey, image=self.img_unchecked)

    def clearCaseSelection(self):
        for key, value in self.caseSelections.items():
            if value == 1:
                self.caseSelections[key] = 0
                self.caseTree.tag_configure(key, image=self.img_unchecked)

    def clearObjectSelection(self):
        for key, value in self.objectSelections.items():
            if value == 1:
                self.objectSelections[key] = 0
                self.objectTree.tag_configure(key, image=self.img_unchecked)

    def clearAllSelection(self):
        self.clearCaseSelection()
        self.clearPropertySelection()
        self.clearObjectSelection()


def hello():
    print("hello!")


if __name__ == "__main__":
    root = tk.Tk()
    root.wm_title("Simulation Summary Viewer")
    app = SimResMainFrame(master=root)
    app.mainloop()
